@@ -63,9 +63,9 @@ logml_weight_bvharhm <- function(param, daily, weekly, monthly, eps = 1e-04, y,
 }
 
 #' Fitting Bayesian VHAR of Minnesota Prior
-#' 
+#'
 #' This function fits BVHAR with Minnesota prior.
-#' 
+#'
 #' @param y Time series data of which columns indicate the variables
 #' @param har Numeric vector for weekly and monthly order. By default, `c(5, 22)`.
 #' @param num_chains Number of MCMC chains
@@ -75,23 +75,22 @@ logml_weight_bvharhm <- function(param, daily, weekly, monthly, eps = 1e-04, y,
 #' @param bayes_spec A BVHAR model specification by [set_bvhar()] (default) or [set_weight_bvhar()].
 #' @param scale_variance Proposal distribution scaling constant to adjust an acceptance rate
 #' @param include_mean Add constant term (Default: `TRUE`) or not (`FALSE`)
-#' @param parallel List the same argument of [optimParallel::optimParallel()]. By default, this is empty, and the function does not execute parallel computation.
 #' @param verbose Print the progress bar in the console. By default, `FALSE`.
 #' @param num_thread Number of threads
-#' @details 
+#' @details
 #' Apply Minnesota prior to Vector HAR: \eqn{\Phi} (VHAR matrices) and \eqn{\Sigma_e} (residual covariance).
-#' 
+#'
 #' \deqn{\Phi \mid \Sigma_e \sim MN(M_0, \Omega_0, \Sigma_e)}
 #' \deqn{\Sigma_e \sim IW(\Psi_0, \nu_0)}
 #' (MN: [matrix normal](https://en.wikipedia.org/wiki/Matrix_normal_distribution), IW: [inverse-wishart](https://en.wikipedia.org/wiki/Inverse-Wishart_distribution))
-#' 
+#'
 #' There are two types of Minnesota priors for BVHAR:
-#' 
+#'
 #' * VAR-type Minnesota prior specified by [set_bvhar()], so-called BVHAR-S model.
 #' * VHAR-type Minnesota prior specified by [set_weight_bvhar()], so-called BVHAR-L model.
 #' @return `bvhar_minnesota()` returns an object `bvharmn` [class].
 #' It is a list with the following components:
-#' 
+#'
 #' \describe{
 #'   \item{coefficients}{Posterior Mean}
 #'   \item{fitted.values}{Fitted values}
@@ -122,19 +121,23 @@ logml_weight_bvharhm <- function(param, daily, weekly, monthly, eps = 1e-04, y,
 #' }
 #' It is also `normaliw` and `bvharmod` class.
 #' @references Kim, Y. G., and Baek, C. (2024). *Bayesian vector heterogeneous autoregressive modeling*. Journal of Statistical Computation and Simulation, 94(6), 1139-1157.
-#' @seealso 
+#' @seealso
 #' * [set_bvhar()] to specify the hyperparameters of BVHAR-S
 #' * [set_weight_bvhar()] to specify the hyperparameters of BVHAR-L
 #' * [summary.normaliw()] to summarize BVHAR model
 #' @examples
 #' # Perform the function using etf_vix dataset
-#' fit <- bvhar_minnesota(y = etf_vix[,1:3])
+#' \dontrun{
+#' fit <- bvhar_minnesota(y = etf_vix[, 1:3])
 #' class(fit)
+#' }
 #' 
 #' # Extract coef, fitted values, and residuals
+#' \dontrun{
 #' coef(fit)
 #' head(residuals(fit))
 #' head(fitted(fit))
+#' }
 #' @order 1
 #' @export
 bvhar_minnesota <- function(y,
@@ -146,7 +149,6 @@ bvhar_minnesota <- function(y,
                             bayes_spec = set_bvhar(),
                             scale_variance = .05,
                             include_mean = TRUE,
-                            parallel = list(),
                             verbose = FALSE,
                             num_thread = 1) {
   if (!all(apply(y, 2, is.numeric))) {
@@ -300,84 +302,87 @@ bvhar_minnesota <- function(y,
     # prior selection-------------------
     lower_vec <- unlist(bayes_spec)
     lower_vec <- as.numeric(lower_vec[grepl(pattern = "lower$", x = names(unlist(bayes_spec)))])
-    lower_vec <- c(lower_vec[2], rep(lower_vec[1], dim_data))
+    # lower_vec <- c(lower_vec[2], rep(lower_vec[1], dim_data))
+    lower_vec <- c(rep(lower_vec[1], dim_data), lower_vec[2])
     upper_vec <- unlist(bayes_spec)
     upper_vec <- as.numeric(upper_vec[grepl(pattern = "upper$", x = names(unlist(bayes_spec)))])
-    upper_vec <- c(upper_vec[2], rep(upper_vec[1], dim_data))
+    # upper_vec <- c(upper_vec[2], rep(upper_vec[1], dim_data))
+    upper_vec <- c(rep(upper_vec[1], dim_data), upper_vec[2])
     is_short <- is.null(bayes_spec$daily)
-    if (is_short) {
-      # delta <- bayes_spec$delta
-      if (length(parallel) > 0) {
-        init_par <-
-          optimParallel(
-            par = c(lambda, psi),
-            fn = logml_bvharhm,
-            lower = lower_vec,
-            upper = upper_vec,
-            hessian = TRUE,
-            delta = bayes_spec$delta,
-            eps = bayes_spec$eps,
-            y = y,
-            har = har,
-            include_mean = include_mean,
-            parallel = parallel
-          )
-      } else {
-        init_par <-
-          optim(
-            par = c(lambda, psi),
-            fn = logml_bvharhm,
-            method = "L-BFGS-B",
-            lower = lower_vec,
-            upper = upper_vec,
-            hessian = TRUE,
-            delta = bayes_spec$delta,
-            eps = bayes_spec$eps,
-            y = y,
-            har = har,
-            include_mean = include_mean
-          )
-      }
-    } else {
-      if (length(parallel) > 0) {
-        init_par <-
-          optimParallel(
-            par = c(lambda, psi),
-            fn = logml_weight_bvharhm,
-            lower = lower_vec,
-            upper = upper_vec,
-            hessian = TRUE,
-            daily = bayes_spec$daily,
-            weekly = bayes_spec$weekly,
-            monthly = bayes_spec$monthly,
-            eps = bayes_spec$eps,
-            y = y,
-            har = har,
-            include_mean = include_mean,
-            parallel = parallel
-          )
-      } else {
-        init_par <-
-          optim(
-            par = c(lambda, psi),
-            fn = logml_weight_bvharhm,
-            method = "L-BFGS-B",
-            lower = lower_vec,
-            upper = upper_vec,
-            hessian = TRUE,
-            daily = bayes_spec$daily,
-            weekly = bayes_spec$weekly,
-            monthly = bayes_spec$monthly,
-            eps = bayes_spec$eps,
-            y = y,
-            har = har,
-            include_mean = include_mean
-          )
-      }
-    }
-    lambda <- init_par$par[1]
-    psi <- init_par$par[2:(1 + dim_data)]
-    hess <- init_par$hessian
+    # if (is_short) {
+    #   # delta <- bayes_spec$delta
+    #   if (length(parallel) > 0) {
+    #     init_par <-
+    #       optimParallel(
+    #         par = c(lambda, psi),
+    #         fn = logml_bvharhm,
+    #         lower = lower_vec,
+    #         upper = upper_vec,
+    #         hessian = TRUE,
+    #         delta = bayes_spec$delta,
+    #         eps = bayes_spec$eps,
+    #         y = y,
+    #         har = har,
+    #         include_mean = include_mean,
+    #         parallel = parallel
+    #       )
+    #   } else {
+    #     init_par <-
+    #       optim(
+    #         par = c(lambda, psi),
+    #         fn = logml_bvharhm,
+    #         method = "L-BFGS-B",
+    #         lower = lower_vec,
+    #         upper = upper_vec,
+    #         hessian = TRUE,
+    #         delta = bayes_spec$delta,
+    #         eps = bayes_spec$eps,
+    #         y = y,
+    #         har = har,
+    #         include_mean = include_mean
+    #       )
+    #   }
+    # } else {
+    #   if (length(parallel) > 0) {
+    #     init_par <-
+    #       optimParallel(
+    #         par = c(lambda, psi),
+    #         fn = logml_weight_bvharhm,
+    #         lower = lower_vec,
+    #         upper = upper_vec,
+    #         hessian = TRUE,
+    #         daily = bayes_spec$daily,
+    #         weekly = bayes_spec$weekly,
+    #         monthly = bayes_spec$monthly,
+    #         eps = bayes_spec$eps,
+    #         y = y,
+    #         har = har,
+    #         include_mean = include_mean,
+    #         parallel = parallel
+    #       )
+    #   } else {
+    #     init_par <-
+    #       optim(
+    #         par = c(lambda, psi),
+    #         fn = logml_weight_bvharhm,
+    #         method = "L-BFGS-B",
+    #         lower = lower_vec,
+    #         upper = upper_vec,
+    #         hessian = TRUE,
+    #         daily = bayes_spec$daily,
+    #         weekly = bayes_spec$weekly,
+    #         monthly = bayes_spec$monthly,
+    #         eps = bayes_spec$eps,
+    #         y = y,
+    #         har = har,
+    #         include_mean = include_mean
+    #       )
+    #   }
+    # }
+    # lambda <- init_par$par[1]
+    # psi <- init_par$par[2:(1 + dim_data)]
+    # hess <- init_par$hessian
+    init_par <- list(par = c(psi, lambda))
     # dummy-----------------------------
     # Yp <- build_ydummy_export(3, psi, lambda, delta, numeric(dim_data), numeric(dim_data), include_mean)
     if (is_short) {
@@ -400,6 +405,10 @@ bvhar_minnesota <- function(y,
       y_dummy = Yp,
       param_prior = bayes_spec,
       param_init = param_init,
+      lower = lower_vec,
+      upper = upper_vec,
+      lag = 3,
+      include_mean = include_mean,
       seed_chain = sample.int(.Machine$integer.max, size = num_chains),
       display_progress = verbose,
       nthreads = num_thread
@@ -506,6 +515,17 @@ bvhar_minnesota <- function(y,
   # S3--------------------------------
   res$call <- match.call()
   res$process <- paste(bayes_spec$process, minnesota_type, sep = "_")
+  if (bayes_spec$prior == "MN_VAR") {
+    bayes_spec$sigma <- res$spec[1:dim_data]
+    bayes_spec$lambda <- res$spec[dim_data + 1]
+    bayes_spec$delta <- res$spec[(dim_data + 2):(2 * dim_data + 1)]
+  } else if (bayes_spec$prior == "MN_VHAR") {
+    bayes_spec$sigma <- res$spec[1:dim_data]
+    bayes_spec$lambda <- res$spec[dim_data + 1]
+    bayes_spec$daily <- res$spec[(dim_data + 2):(2 * dim_data + 1)]
+    bayes_spec$weekly <- res$spec[(2 * dim_data + 2):(3 * dim_data + 1)]
+    bayes_spec$daily <- res$spec[(3 * dim_data + 2):(4 * dim_data + 1)]
+  }
   res$spec <- bayes_spec
   # class(res) <- c("bvharmn", "normaliw", "bvharmod")
   if (minnesota_type == "MN_Hierarchical") {
