@@ -3,11 +3,13 @@
 
 #include "./spillover.h"
 
+namespace baecon {
 namespace bvhar {
 
 using RegSpillover = McmcVarSpillover<LdltRecords>;
 using RegVharSpillover = McmcVharSpillover<LdltRecords>;
 
-}; // namespace bvhar
+} // namespace bvhar
+} // namespace baecon
 
 #endif // BVHAR_BAYES_TRIANGULAR_REGSPILLOVER_H
