@@ -38,13 +38,13 @@ test_that("BVAR evaluation", {
   skip_on_cran()
   
   test_lag <- 3
-  etf_ncol <- ncol(etf_vix)
+  etf_ncol <- ncol(etf_vix[1:50, 1:3])
   bvar_spec <- set_bvar(
-    sigma = apply(etf_vix, 2, sd),
+    sigma = apply(etf_vix[1:50, 1:3], 2, sd),
     lambda = .2,
     delta = rep(.1, etf_ncol)
   )
-  fit_test_bvar <- bvar_minnesota(etf_vix, test_lag, bayes_spec = bvar_spec)
+  fit_test_bvar <- bvar_minnesota(etf_vix[1:50, 1:3], test_lag, bayes_spec = bvar_spec)
   expect_s3_class(fit_test_bvar, "bvarmn")
   
   expect_s3_class(logLik(fit_test_bvar), "logLik")
@@ -57,13 +57,13 @@ test_that("BVHAR-S evaluation", {
   skip_on_cran()
   
   har <- c(5, 22)
-  etf_ncol <- ncol(etf_vix)
+  etf_ncol <- ncol(etf_vix[1:50, 1:3])
   vhar_spec <- set_bvhar(
-    sigma = apply(etf_vix, 2, sd),
+    sigma = apply(etf_vix[1:50, 1:3], 2, sd),
     lambda = .2,
     delta = rep(.1, etf_ncol)
   )
-  fit_test_bvhar_s <- bvhar_minnesota(etf_vix, har = har, bayes_spec = vhar_spec)
+  fit_test_bvhar_s <- bvhar_minnesota(etf_vix[1:50, 1:3], har = har, bayes_spec = vhar_spec)
   expect_s3_class(fit_test_bvhar_s, "bvharmn")
   
   expect_s3_class(logLik(fit_test_bvhar_s), "logLik")
@@ -76,16 +76,16 @@ test_that("BVHAR-L evaluation", {
   skip_on_cran()
   
   har <- c(5, 22)
-  etf_ncol <- ncol(etf_vix)
+  etf_ncol <- ncol(etf_vix[1:50, 1:3])
   vhar_spec <- set_weight_bvhar(
-    sigma = apply(etf_vix, 2, sd),
+    sigma = apply(etf_vix[1:50, 1:3], 2, sd),
     lambda = .2,
     daily = rep(.3, etf_ncol),
     weekly = rep(.2, etf_ncol),
     monthly = rep(.1, etf_ncol)
   )
   
-  fit_test_bvhar_s <- bvhar_minnesota(etf_vix, har = har, bayes_spec = vhar_spec)
+  fit_test_bvhar_s <- bvhar_minnesota(etf_vix[1:50, 1:3], har = har, bayes_spec = vhar_spec)
   expect_s3_class(fit_test_bvhar_s, "bvharmn")
   
   expect_s3_class(logLik(fit_test_bvhar_s), "logLik")
