@@ -70,7 +70,7 @@ test_that("Expanding windows - OLS VHAR", {
   expect_s3_class(var_expand, "bvharcv")
 })
 
-help_var_bayes_roll <- function(bayes_spec, cov_spec, sparse, is_exogen = FALSE) {
+help_var_bayes_roll <- function(bayes_spec, cov_spec, sparse, is_exogen = FALSE, use_fit = TRUE) {
   etf_train <- etf_vix[1:50, 1:2]
   etf_test <- etf_vix[51:53, 1:2]
   etf_exogen_train <- NULL
@@ -85,6 +85,7 @@ help_var_bayes_roll <- function(bayes_spec, cov_spec, sparse, is_exogen = FALSE)
     etf_train,
     p = 1,
     exogen = etf_exogen_train,
+    factor_spec = set_factor(size_factor = 1),
     num_iter = 3,
     num_burn = 0,
     coef_spec = bayes_spec,
@@ -93,10 +94,10 @@ help_var_bayes_roll <- function(bayes_spec, cov_spec, sparse, is_exogen = FALSE)
     include_mean = TRUE
   )
   set.seed(1)
-  forecast_roll(fit_test, 1, etf_test, newxreg = etf_exogen_test, stable = FALSE, sparse = sparse, lpl = TRUE)
+  forecast_roll(fit_test, 1, etf_test, newxreg = etf_exogen_test, stable = FALSE, sparse = sparse, lpl = TRUE, use_fit = use_fit)
 }
 
-help_vhar_bayes_roll <- function(bayes_spec, cov_spec, sparse, is_exogen = FALSE) {
+help_vhar_bayes_roll <- function(bayes_spec, cov_spec, sparse, is_exogen = FALSE, use_fit = TRUE) {
   etf_train <- etf_vix[1:50, 1:2]
   etf_test <- etf_vix[51:53, 1:2]
   etf_exogen_train <- NULL
@@ -110,6 +111,7 @@ help_vhar_bayes_roll <- function(bayes_spec, cov_spec, sparse, is_exogen = FALSE
   fit_test <- vhar_bayes(
     etf_train,
     exogen = etf_exogen_train,
+    factor_spec = set_factor(size_factor = 1),
     num_iter = 3,
     num_burn = 0,
     coef_spec = bayes_spec,
@@ -118,10 +120,10 @@ help_vhar_bayes_roll <- function(bayes_spec, cov_spec, sparse, is_exogen = FALSE
     include_mean = TRUE
   )
   set.seed(1)
-  forecast_roll(fit_test, 1, etf_test, newxreg = etf_exogen_test, stable = FALSE, sparse = sparse, lpl = TRUE)
+  forecast_roll(fit_test, 1, etf_test, newxreg = etf_exogen_test, stable = FALSE, sparse = sparse, lpl = TRUE, use_fit = use_fit)
 }
 
-help_var_bayes_expand <- function(bayes_spec, cov_spec, sparse, is_exogen = FALSE) {
+help_var_bayes_expand <- function(bayes_spec, cov_spec, sparse, is_exogen = FALSE, use_fit = TRUE) {
   etf_train <- etf_vix[1:50, 1:2]
   etf_test <- etf_vix[51:53, 1:2]
   etf_exogen_train <- NULL
@@ -136,6 +138,7 @@ help_var_bayes_expand <- function(bayes_spec, cov_spec, sparse, is_exogen = FALS
     etf_train,
     p = 1,
     exogen = etf_exogen_train,
+    factor_spec = set_factor(size_factor = 1),
     num_iter = 3,
     num_burn = 0,
     coef_spec = bayes_spec,
@@ -144,10 +147,10 @@ help_var_bayes_expand <- function(bayes_spec, cov_spec, sparse, is_exogen = FALS
     include_mean = TRUE
   )
   set.seed(1)
-  forecast_expand(fit_test, 1, etf_test, newxreg = etf_exogen_test, stable = FALSE, sparse = sparse, lpl = TRUE)
+  forecast_expand(fit_test, 1, etf_test, newxreg = etf_exogen_test, stable = FALSE, sparse = sparse, lpl = TRUE, use_fit = use_fit)
 }
 
-help_vhar_bayes_expand <- function(bayes_spec, cov_spec, sparse, is_exogen = FALSE) {
+help_vhar_bayes_expand <- function(bayes_spec, cov_spec, sparse, is_exogen = FALSE, use_fit = TRUE) {
   etf_train <- etf_vix[1:50, 1:2]
   etf_test <- etf_vix[51:53, 1:2]
   etf_exogen_train <- NULL
@@ -161,6 +164,7 @@ help_vhar_bayes_expand <- function(bayes_spec, cov_spec, sparse, is_exogen = FAL
   fit_test <- vhar_bayes(
     etf_train,
     exogen = etf_exogen_train,
+    factor_spec = set_factor(size_factor = 1),
     num_iter = 3,
     num_burn = 0,
     coef_spec = bayes_spec,
@@ -169,7 +173,7 @@ help_vhar_bayes_expand <- function(bayes_spec, cov_spec, sparse, is_exogen = FAL
     include_mean = TRUE
   )
   set.seed(1)
-  forecast_expand(fit_test, 1, etf_test, newxreg = etf_exogen_test, stable = FALSE, sparse = sparse, lpl = TRUE)
+  forecast_expand(fit_test, 1, etf_test, newxreg = etf_exogen_test, stable = FALSE, sparse = sparse, lpl = TRUE, use_fit = use_fit)
 }
 
 test_that("Rolling windows - VAR-Minn-LDLT", {
@@ -194,6 +198,9 @@ test_that("Rolling windows - VAR-HS-LDLT", {
   test_roll_sparse <- help_var_bayes_roll(set_horseshoe(), set_ldlt(), TRUE)
   test_roll_dense_x <- help_var_bayes_roll(set_horseshoe(), set_ldlt(), FALSE, TRUE)
   test_roll_sparse_x <- help_var_bayes_roll(set_horseshoe(), set_ldlt(), TRUE, TRUE)
+  expect_no_error({
+    help_var_bayes_roll(set_horseshoe(), set_ldlt(), FALSE, TRUE, FALSE)
+  })
 
   expect_s3_class(test_roll_dense, "predbvhar_roll")
   expect_s3_class(test_roll_dense, "bvharcv")
@@ -269,6 +276,9 @@ test_that("Rolling windows - VHAR-Minn-LDLT", {
   test_roll_sparse <- help_vhar_bayes_roll(set_bvhar(), set_ldlt(), TRUE)
   test_roll_dense_x <- help_vhar_bayes_roll(set_bvhar(), set_ldlt(), FALSE, TRUE)
   test_roll_sparse_x <- help_vhar_bayes_roll(set_bvhar(), set_ldlt(), TRUE, TRUE)
+  expect_no_error({
+    help_vhar_bayes_roll(set_bvhar(), set_ldlt(), FALSE, TRUE, FALSE)
+  })
 
   expect_s3_class(test_roll_dense, "predbvhar_roll")
   expect_s3_class(test_roll_dense, "bvharcv")
@@ -584,6 +594,9 @@ test_that("Expanding windows - VAR-Minn-LDLT", {
   test_expand_sparse <- help_var_bayes_expand(set_bvar(), set_ldlt(), TRUE)
   test_expand_dense_x <- help_var_bayes_expand(set_bvar(), set_ldlt(), FALSE, TRUE)
   test_expand_sparse_x <- help_var_bayes_expand(set_bvar(), set_ldlt(), TRUE, TRUE)
+  expect_no_error({
+    help_var_bayes_expand(set_bvar(), set_ldlt(), FALSE, TRUE, FALSE)
+  })
 
   expect_s3_class(test_expand_dense, "predbvhar_expand")
   expect_s3_class(test_expand_dense, "bvharcv")
@@ -689,6 +702,9 @@ test_that("Expanding windows - VHAR-Minn-LDLT", {
   test_expand_sparse <- help_vhar_bayes_expand(set_bvhar(), set_ldlt(), TRUE)
   test_expand_dense_x <- help_vhar_bayes_expand(set_bvhar(), set_ldlt(), FALSE, TRUE)
   test_expand_sparse_x <- help_vhar_bayes_expand(set_bvhar(), set_ldlt(), TRUE, TRUE)
+  expect_no_error({
+    help_vhar_bayes_expand(set_bvhar(), set_ldlt(), FALSE, TRUE, FALSE)
+  })
 
   expect_s3_class(test_expand_dense, "predbvhar_expand")
   expect_s3_class(test_expand_dense, "bvharcv")
